//----------------------------------------------------------------------
// PennypackJavaAuthCas/Cgi.java
// Author: Bob Dondero
//----------------------------------------------------------------------

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.StringTokenizer;
import java.util.HashMap;
import java.net.URLDecoder;

public class Cgi
{
   public static final int GET = 0;
   public static final int POST = 1;

   public static int getRequestMethod()
   {
      String requestMethod = System.getenv("REQUEST_METHOD");
      if (requestMethod.toLowerCase().equals("get"))
         return GET;
      return POST;
   }

   public static HashMap<String,String> parseFormFields(String buffer)
   {
      HashMap<String,String> hashMap = new HashMap<String,String>();

      //  Split the buffer into key/value pairs.
      StringTokenizer pairTokenizer =
         new StringTokenizer(buffer, "&");

      while (pairTokenizer.hasMoreTokens())
      {
         String pair = pairTokenizer.nextToken();

         // Split the key/value pair into a key and value.
         StringTokenizer keyValTokenizer =
            new StringTokenizer(pair, "=");

         String key = "";
         if (keyValTokenizer.hasMoreTokens())
            key = keyValTokenizer.nextToken();

         String value = "";
         if (keyValTokenizer.hasMoreTokens())
            value = keyValTokenizer.nextToken();

         try
         {
            key = URLDecoder.decode(key, "UTF-8");
            value = URLDecoder.decode(value, "UTF-8");
         }
         catch (Exception e) { System.err.println(e); }

         hashMap.put(key, value);
      }

      return hashMap;
   }

   public static HashMap<String,String> getFormFields()
   {
      String buffer = "";

      if (getRequestMethod() == GET)
         buffer = System.getenv("QUERY_STRING");
      else  // getRequestMethod() == POST
      {
         InputStreamReader isr = new InputStreamReader(System.in);
         BufferedReader br = new BufferedReader(isr);
         String line;
         try
         {
            while ((line = br.readLine()) != null)
               buffer += line;
         }
         catch (Exception e) { System.err.println(e); }
      }
      return parseFormFields(buffer);
   }

   public static HashMap<String,String> parseCookieFields(String buffer)
   {
      HashMap<String,String> hashMap = new HashMap<String,String>();

      // Parse the buffer into key/value pairs.
      // Will parse incorrectly if a cookie value contains a semicolon.
      // So make sure your application avoids such cookie values.
      // For example, your application could URL-encode cookie values.
      StringTokenizer pairTokenizer = new StringTokenizer(buffer, ";");

      while (pairTokenizer.hasMoreTokens())
      {
         String pair = pairTokenizer.nextToken().trim();

         // Split the key/value pair into a key and value.
         String key = "";
         String value = "";
         int index = pair.indexOf('=');
         if (index == -1)
            key = pair;
         else
         {
            key = pair.substring(0, index);
            value = pair.substring(index+1);
         }
         hashMap.put(key, value);
      }

      return hashMap;
   }

   public static HashMap<String,String> getCookies()
   {
      String buffer = System.getenv("HTTP_COOKIE");
      if (buffer == null)
         return new HashMap<String,String>();

      return parseCookieFields(buffer);
   }
}
